@@ -1,3 +1,4 @@
+from datetime import timedelta
 from collections.abc import Mapping
 import os
 from typing import Awaitable, Callable, Iterable
@@ -13,13 +14,14 @@ from beneath.admin.secrets import Secrets
 from beneath.admin.services import Services
 from beneath.admin.streams import Streams
 from beneath.admin.users import Users
-from beneath.connection import Connection
+from beneath.connection import Connection, GraphQLError
 from beneath.config import (
   DEFAULT_READ_ALL_MAX_BYTES,
   DEFAULT_READ_BATCH_SIZE,
   DEFAULT_SUBSCRIBE_CONCURRENT_CALLBACKS,
   DEFAULT_SUBSCRIBE_PREFETCHED_RECORDS,
 )
+from beneath.utils import StreamQualifier
 
 
 class Client:
@@ -47,26 +49,49 @@ class Client:
       raise TypeError("secret must be a string")
     return secret.strip()
 
-  async def find_stream(self, project: str = None, stream: str = None, stream_id: str = None) -> Stream:
-    stream = Stream(client=self, project=project, stream=stream, stream_id=stream_id)
+  async def find_stream(self, path: str) -> Stream:
+    qualifier = StreamQualifier.from_path(path)
+    stream = Stream(client=self, qualifier=qualifier)
     # pylint: disable=protected-access
     await stream._ensure_loaded()
     return stream
 
+  async def stage_stream(
+    self,
+    path: str,
+    schema: str,
+    retention: timedelta = None,
+    create_primary_instance: bool = True,
+  ) -> Stream:
+    qualifier = StreamQualifier.from_path(path)
+    data = await self.admin.streams.stage(
+      organization_name=qualifier.organization,
+      project_name=qualifier.project,
+      stream_name=qualifier.stream,
+      schema_kind="GraphQL",
+      schema=schema,
+      retention_seconds=retention.seconds if retention else None,
+      create_primary_instance=create_primary_instance,
+    )
+    stream = Stream(client=self, qualifier=qualifier)
+    # pylint: disable=protected-access
+    await stream._ensure_loaded(prefetched=data)
+    return stream
+
   # EASY HELPERS
 
   async def easy_read(
     self,
-    project: str,
-    stream: str,
-    where: str = None,
+    stream_path: str,
+    # pylint: disable=redefined-builtin
+    filter: str = None,
     to_dataframe=True,
     batch_size=DEFAULT_READ_BATCH_SIZE,
     max_bytes=DEFAULT_READ_ALL_MAX_BYTES,
     warn_max=True,
   ) -> Iterable[Mapping]:
-    stream = await self.find_stream(project=project, stream=stream)
-    cursor = await stream.query(where=where)
+    stream = await self.find_stream(path=stream_path)
+    cursor = await stream.query_index(filter=filter)
     res = await cursor.fetch_all(
       max_bytes=max_bytes,
       batch_size=batch_size,
@@ -77,15 +102,15 @@ class Client:
 
   async def easy_process_once(
     self,
-    project: str,
-    stream: str,
+    stream_path: str,
     callback: Callable[[Mapping], Awaitable[None]],
-    where: str = None,
+    # pylint: disable=redefined-builtin
+    filter: str = None,
     max_prefetched_records=DEFAULT_SUBSCRIBE_PREFETCHED_RECORDS,
     max_concurrent_callbacks=DEFAULT_SUBSCRIBE_CONCURRENT_CALLBACKS,
   ):
-    stream = await self.find_stream(project=project, stream=stream)
-    cursor = await stream.query(where=where)
+    stream = await self.find_stream(path=stream_path)
+    cursor = await stream.query_index(filter=filter)
     await cursor.subscribe_replay(
       callback=callback,
       max_prefetched_records=max_prefetched_records,
@@ -94,15 +119,13 @@ class Client:
 
   async def easy_process_forever(
     self,
-    project: str,
-    stream: str,
+    stream_path: str,
     callback: Callable[[Mapping], Awaitable[None]],
-    where: str = None,
     max_prefetched_records=DEFAULT_SUBSCRIBE_PREFETCHED_RECORDS,
     max_concurrent_callbacks=DEFAULT_SUBSCRIBE_CONCURRENT_CALLBACKS,
   ):
-    stream = await self.find_stream(project=project, stream=stream)
-    cursor = await stream.query(where=where)
+    stream = await self.find_stream(path=stream_path)
+    cursor = await stream.query_index()
     await cursor.subscribe_replay(
       callback=callback,
       max_prefetched_records=max_prefetched_records,
